import { AnyFun } from "queenjs/typing";
import { EditorModule } from "../..";
import { HistoryController } from "./HistoryController";
import { createProxyStore } from "./createProxyStore";

export class HistoryCtrl {
  history: HistoryController;
  opStatus = false;
  proxyStore: any;

  constructor(protected module: EditorModule, historyTotal = 50) {
    this.history = new HistoryController(historyTotal);
    this.proxyStore = createProxyStore(module.store, this.history);
  }

  applyActions(actNames: string[]) {
    const actions: any = this.module.actions;
    actNames.forEach((actName) => {
      const action = actions[actName];
      actions[actName] = proxyAction.bind(this, action);
    });
  }
}

async function proxyAction(this: HistoryCtrl, action: AnyFun, ...args: any[]) {
  if (this.opStatus) {
    return await action(...args);
  }
  const { module, proxyStore, history } = this;
  const { store } = module;
  try {
    module.store = proxyStore;
    history.group = true;
    await action(...args);
    history.group = false;
  } catch (error) {
    if (history.groupQueue.length) {
      history.group = false;
      history.undo();
      history.queue.pop();
      history.state.lenth--;
    } else {
      history.group = false;
    }
  } finally {
    module.store = store;
  }
}
